"""
    This module contains all defaults available for when
    the application importing the lib doesnt set values
"""
DASHBOARD_URL_PREFIX = 'admindashboard'
POST_LOGIN_PAGE = 'users'
SIDEBAR_TEMPLATE = 'dashboard/operations_sidebar'